declare const DeleteMessage = "\n  local groupSetKey = KEYS[1];\n  local messageQueueKey = KEYS[2];\n\n  local groupId = ARGV[1];\n  local messageData = ARGV[2];\n\n  local removedMsgsCount = redis.call('ZREM', messageQueueKey, messageData);\n\n  --[[\n    local len = redis.call('ZCARD', messageQueueKey);\n\n    if (removedMsgsCount > 0) then\n      if (len > 0) then\n        redis.call('ZINCRBY', groupSetKey, -1, groupId);\n      else\n        redis.call('ZREM', groupSetKey, groupId);\n      end\n    end\n\n    return { removedMsgsCount, len };\n  ]]--\n\n  return { removedMsgsCount };\n";
declare const AddGroupAndMessageToQueue = "\n  -- Processing group\n  local groupStreamKey = KEYS[1];\n  local groupSetKey = KEYS[2];\n\n  -- Message data\n  local priorityMessageQueueKey = KEYS[3];\n\n  -- Processing group\n  local groupId = ARGV[1];\n\n  -- Message data\n  local messagePriority = ARGV[2];\n  local messageData = ARGV[3];\n\n  -- Push message data into priority queue\n  local addedMsgCount = redis.call('ZADD', priorityMessageQueueKey, messagePriority, messageData);\n  local newScoreOfGroup = 0;\n\n  if (addedMsgCount > 0) then\n    -- Add to unlocked group set\n    newScoreOfGroup = tonumber(redis.call('ZINCRBY', groupSetKey, addedMsgCount, groupId));\n\n    if (newScoreOfGroup == 1) then\n      -- Add to stream of groups for processing\n      redis.call('XADD', groupStreamKey, '*', 'group', groupId);\n    end\n  end\n\n  return { addedMsgCount, newScoreOfGroup, messageData };\n";
declare const ClaimTimedOutGroup = "\n  -- Processing group\n  local groupStreamKey = KEYS[1];\n\n  -- Consumer group & consumer\n  local consumerGroupId = ARGV[1];\n  local consumerId = ARGV[2];\n\n  -- Timeout milliseconds\n  local timeoutMs = tonumber(ARGV[3]);\n\n  local pending_msgs = redis.call('XPENDING', groupStreamKey, consumerGroupId, '-', '+', '1');\n\n  if (#(pending_msgs) > 0) then\n    -- Claim message by Id for this consumer\n    local pending_msg_id = pending_msgs[1][1];\n    local pending_msg_consumer_id = pending_msgs[1][2];\n    local pending_msg_idle_ms = tonumber(pending_msgs[1][3]);\n\n    if (pending_msg_idle_ms >= timeoutMs) then\n      local claimed_msgs = redis.call('XCLAIM', groupStreamKey, consumerGroupId, consumerId, 0, pending_msg_id);\n\n      if (#(claimed_msgs) > 0) then\n        local msg_id = claimed_msgs[1][1];\n        local msg_content = claimed_msgs[1][2];\n\n        -- Check remaining pending msgs for this consumer, if no more msgs are pending, we can delete the consumer\n        local pending_msgs_same_consumer = redis.call('XPENDING', groupStreamKey, consumerGroupId, '-', '+', '1', pending_msg_consumer_id);\n\n        if (#(pending_msgs_same_consumer) == 0) then\n          -- Delete consumer which has timed out\n          redis.call('XGROUP', 'DELCONSUMER', groupStreamKey, consumerGroupId, pending_msg_consumer_id)\n        end\n\n        local group_id = msg_content[2];\n\n        return { msg_id, group_id };\n      end\n    end\n  end\n\n  return {};\n";
declare const UnlockGroup = "\n  -- Processing group\n  local groupStreamKey = KEYS[1];\n  local groupSetKey = KEYS[2];\n\n  -- Message data\n  local priorityMessageQueueKey = KEYS[3];\n\n  -- Consumer group & consumer\n  local consumerGroupId = ARGV[1];\n  local consumerId = ARGV[2];\n  local messageId = ARGV[3];\n  local groupId = ARGV[4];\n\n  redis.call('XACK', groupStreamKey, consumerGroupId, messageId);\n  redis.call('XDEL', groupStreamKey, messageId);\n  redis.call('XGROUP', 'DELCONSUMER', groupStreamKey, consumerGroupId, consumerId);\n\n  local numMsgs = tonumber(redis.call('ZCARD', priorityMessageQueueKey));\n\n  if (numMsgs == 0) then\n    -- No msgs, remove group set key\n    redis.call('ZREM', groupSetKey, groupId);\n\n    return { 1, numMsgs };\n  else\n    -- Has msgs, update group set score to reflect current number of messages\n    redis.call('ZADD', groupSetKey, 'CH', numMsgs, groupId);\n\n    -- Has msgs, re-add group to the tail of the stream\n    redis.call('XADD', groupStreamKey, '*', 'group', groupId);\n\n    return { 0, numMsgs };\n  end\n";
declare const GetMetrics = "\n  -- Processing group\n  local groupStreamKey = KEYS[1];\n  local groupSetKey = KEYS[2];\n\n  -- Message data\n  local consumerGroupId = ARGV[1];\n\n  -- Processing group\n  local topMessageGroupsLimit = tonumber(ARGV[2]);\n\n  local sum=0\n  local z=redis.call('ZRANGE', groupSetKey, 0, -1, 'WITHSCORES')\n\n  for i=2, #z, 2 do \n      sum=sum+z[i]\n  end\n\n  local topGroups = redis.call('ZREVRANGE', groupSetKey, 0, topMessageGroupsLimit, 'WITHSCORES')\n\n  local consumers = redis.call('XINFO', 'CONSUMERS', groupStreamKey, consumerGroupId)\n\n  return {\n    redis.call('XLEN', groupStreamKey),\n    redis.call('ZCARD', groupSetKey),\n    #(consumers),\n    sum,\n    topGroups\n  }\n";
export { DeleteMessage, AddGroupAndMessageToQueue, ClaimTimedOutGroup, UnlockGroup, GetMetrics };
